#!/bin/bash

# this file should be sourced by all shell scripts.
# unexport evil environment variables (must also be in top-level Makefile)
set +e
unset LC_COLLATE
unset LC_CTYPE
unset LC_MONETARY
unset LC_NUMERIC
unset LC_TIME
unset LC_ALL
unset LANG
unset LANGUAGE
unset LOCPATH
unset NLSPATH
unset TZDIR

umask 022
set -e


# search top midori directory
while [ ! -f apps/mlbuild*.mlz ] ; do
        cd ..
        if [ `pwd` = / ] ; then
            echo "The top level of your Midori Linux build tree could not be found,"
            echo "aborting to avoid serious problems"
            exit 1
        fi
done

# sanity check
if [ ! -f Makefile -o ! -x $SCRIPTS/functions.sh -o ! -d apps ] ; then
        echo "Some crucial files or directories seem to be missing from your "
        echo "midori linux distribution. Aborting..."
        set +e
        ls -l Makefile
        ls -l $SCRIPTS/functions.sh
        ls -ld apps
        exit 1
fi

. .config.sh

export BUILD_OPTIONS="$*"

# the following function makes life much easier for you, provided that you
# define the necessary functions. Your build script should first define
# the variables shown below, then overwrite the functions necessary.
# Note that cwd==$BUILD when calling any of the functions.

function ml_reset() {
        # Set to yes if you want statistics for this package.
        # Usually a good thing for packages storing data in the
        # root directory, for others its a waste of time.
        # Default is yes
        STATISTICS=yes
        # list of files stored in download that need to be unpacked
        # (unnecessary if you define your own ml_untar function, in this
        # case simply define a name that will be used to name patched).
        # if TARFILES is emtpy, it assumes TARFILES=$PACKAGE.tar.gz
        TARFILES=
        # list of patchfiles
        PATCHFILES=
        # patch argument
        PATCHARG=-p0
        # options for configure
        CONFIGOPTIONS=
        # options for compilation
        COMPILEOPTIONS="CC=\"$CC\" GCC=\"$GCC\""
        # list of documentation files and menus
        DOCFILES=
        # runtime test dirs
        RUNTIMETESTS=
        # List of configuration menus for runtime configuration
        MLCONFIG_MENUS=
        # list of functions to call in this order (without the leading ml_)
        # Unless a function name is listed in EXECUTE_ALWAYS it is only executed
        # if there is no marker file $BUILD/$PACKAGE/.functionname. Functions listed
        # in DANGEROUS_FUNCTIONS fail if the build directory contains a .precious file.
        #
        # For each function not listed in EXECUTE_ALWAYS  you can also set to variables
        # with the name FUNCTION_OPTIONS and FUNCTION_FILES (note the uppercase
        # FUNCTION). FUNCTION_OPTIONS should be set to all options that, if changed,
        # should trigger function() again.  FUNCTION_FILES lists all files that,
        # if changed, would trigger function().
        #
        # FUNCTIONS must always contain a function called 'installconfig'. If you use
        # the default one you also need to use 'findfiles'
        FUNCTIONS="findfiles untar patchit installconfig configure compile installlocal installroot"
        EXECUTE_ALWAYS="findfiles installconfig installroot clean"
        DANGEROUS_FUNCTIONS="untar patchit"
        # list options that, if changed, require a 'make cleanimage' instead
        # of just a 'make clean'
        CLEANIMAGE_OPTIONS=

        # redefine all functions
        function ml_findfiles() { ml_findfiles_default ; }
        function ml_untar() { ml_untar_default ; }
        function ml_patchit() { ml_patchit_default ; }
        function ml_configure() { ml_configure_default ; }
        function ml_compile() { ml_compile_default ; }
        function ml_installconfig() { ml_installconfig_default ; }
        function ml_clean() { ml_clean_default ; }

        # install files into the tools directory
        function ml_installlocal() { : ; }
        # install files into the root directory
        function ml_installroot() { ml_installroot_default ; }

        # all the vars that define when a package gets rebuild
        UNTAR_FILES=
        UNTAR_OPTIONS=
        PATCHIT_FILES=
        PATCHIT_OPTIONS=
        CONFIGURE_FILES=
        CONFIGURE_OPTIONS=
        COMPILE_FILES=
        COMPILE_OPTIONS=
        INSTALLLOCAL_FILES=
        INSTALLLOCAL_OPTIONS=
}

# define an initialization script.
# $1: partition where it should be started, usually one of
#     /  /usr  or  /usr/X11R6
# $2: script type, must be 'init', 'suspend', 'resume', 'stop', 'die'
# $3: Execution priority (two digit number by convention)
# $4: file that contains script or script itself, each argument
#     will be written to a new line.
function ml_initscript() {
	initpart=`echo $1 | tr / ,`
	if [ ! -d $INITS/$2 ] ; then
		echo illegal script type $2, must be one of \"init\", \"suspend\", \"resume\", \"stop\", \"die\"
		exit 1
	fi
	mkdir -p $INITS/$2/$initpart
	initname="$INITS/$2/$initpart/$3_$PACKAGE"
	if [ -f "$4" ] ; then
		if cmp -s $4 $initname ; then
			:
		else
			cp $4 $initname
			rm -f $ROOT/sbin/init # trigger init script creation
		fi
	else
		shift
		shift
		shift
                local i
		for i do
			echo "$i"
		done > $initname.tmp
		if cmp -s $initname.tmp $initname ; then
			rm $initname.tmp
		else
			mv -f $initname.tmp $initname
			rm -f $ROOT/sbin/init # trigger init script creation
		fi
	fi
}

# write arguments to log file
function ml_log() {
        local d
        d=`date`
	echo $d: $PACKAGE: "$@" >> $CACHE/log
	[ ! -d $BUILD/$PACKAGE ] || echo $d: $PACKAGE: "$@" >> $BUILD/$PACKAGE/log
}


# Tarfiles are looked up in the packages dir, and then in $TARS
function ml_findfiles_default() {
        if [ "$TARFILES" = "" ] ; then
                if [ -f $BUILD/$PACKAGE/$PACKAGE.tar.gz ] ; then
                        TARFILES=$PACKAGE.tar.gz
                elif [ -f $BUILD/$PACKAGE/$PACKAGE.tar.bz2 ] ; then
                        TARFILES=$PACKAGE.tar.bz2
                elif [ -f $BUILD/$PACKAGE/$PACKAGE.tgz ] ; then
                        TARFILES=$PACKAGE.tgz
                elif [ -f $TARS/$PACKAGE.tar.gz ] ; then
                        TARFILES=$PACKAGE.tar.gz
                elif [ -f $TARS/$PACKAGE.tar.bz2 ] ; then
                        TARFILES=$PACKAGE.tar.bz2
                elif [ -f $TARS/$PACKAGE.tgz ] ; then
                        TARFILES=$PACKAGE.tgz
                fi

## it seems that UNTAR_FILES isn't used at all ...
#                if [ -f  $BUILD/$PACKAGE/$TARFILES ] ; then
#                        UNTAR_FILES="$UNTAR_FILES $BUILD/$PACKAGE/$TARFILES"
#                else
#                        TARFILES=
#                fi
        fi
        if [ "$PATCHFILES" = "" ] ; then
                PATCHFILES=`cd $BUILD/$PACKAGE ; find $BUILD/$PACKAGE -maxdepth 2 -name $PACKAGE.patch\*`
        fi
        local i
        for i in $PATCHFILES
        do
                if [ -f $BUILD/$PACKAGE/$i ] ; then
                        PATCHIT_FILES="$PATCHIT_FILES $BUILD/$PACKAGE/$i"
                else
                        PATCHIT_FILES="$PATCHIT_FILES $i"
                fi
        done
        if [ "$DOCFILES" = "" -a -d $BUILD/$PACKAGE/docs ] ; then
                DOCFILES=`find $BUILD/$PACKAGE/docs -maxdepth 2 -name \*.txt -o -name \*.html -o -name \*.gif -o -name \*.menu`
        fi
        if [ "$RUNTIMETESTS" = "" -a -d $BUILD/$PACKAGE/tests/runtime ] ; then
                cd $BUILD/$PACKAGE/tests/runtime
                for i in `echo */runtest` ; do
                        if [ -x $i ] ; then
                                RUNTIMETESTS="$RUNTIMETESTS `dirname $i`"
                        fi
                done
        fi
}

function ml_untar_default() {
        # special case for *.mlz archives
        local i tarcmd
        if [ -d $PACKAGE/$PACKAGE ] ; then
                rm -rf $PACKAGE/$PACKAGE
        fi
        for i in $TARFILES
        do
                cd $PACKAGE
                if [ `basename $i .gz` != `basename $i` ] ; then
                        tarcmd="tar xzf "
                elif [ `basename $i .bz2` != `basename $i` ] ; then
			tarcmd="tar --use-compress-program bzip2 -xf "
                elif [ `basename $i .tgz` != `basename $i` ] ; then
                        tarcmd="tar xzf "
                else
                        echo "cannot find tarfile $i"
                        exit 1
                fi
		if [ -f $i ] ; then
			# tarfile is inside package
			eval $tarcmd $i
		elif [ -f $TARS/$i ] ; then
			# tarfile is in seperate tar directory
			eval $tarcmd $TARS/$i
		else
			echo "Cannot find tarfile $i. Check if it is missing in $TARS"
			exit 1
		fi
                cd ..
        done

}

function ml_patchit_default() {
        cd $PACKAGE
        local i
        for i in $PATCHFILES
        do
                if [ `basename $i .gz` != `basename $i` ] ; then
                        gzip -d < $i
                elif [ `basename $i .bz2` != `basename $i` ] ; then
                        bzip2 -d < $i
                else
                        cat $i
                fi | patch $PATCHARG
        done
}

function ml_configure_default() {
        cd $BUILD/$PACKAGE
        if [ -x $PACKAGE/configure ] ; then
                cd $PACKAGE
                eval `echo ./configure $CONFIGOPTIONS`
        fi
}

function ml_compile_default() {
        cd $BUILD/$PACKAGE
        if [ -d $PACKAGE ] ; then
                cd $PACKAGE
                [ ! -f Makefile ] || eval `echo $JMAKE $COMPILEOPTIONS`
        fi
}

function mli_add_config() {
        local f i
        f=$1
        shift
        for i do
                egrep -q "\<$i\>" $f || echo $i >> $f
        done
}

function mli_remove_config() {
        local f i n
        f=$1
        shift
        n=`cat $f`
        {
                echo "$n"
                for i do
                        echo $i
                        echo $i
                done
        } | sort | uniq -u > $f
}

function mli_create_index() {
        (
                cd $TOPDIR/docs/packages
                echo "<html><head><title>Midori Documentation</title></head><body bgcolor=ffffff>"
                echo "<h1>Midori&nbsp;Documentation</h1><dl>"
                grep -H '[ \t]' `find . -maxdepth 2 -name \*.menu -print` | sort -t: +1 | while read a
                do
                        f=`echo $a | awk -F: '{ print $1 }'`
                        m=`echo $a | awk -F: '{ print $2 }'`
                        n=`echo $a | awk -F: '{ print $3 }'`
                        if [ "$m" != "$lastm" ] ; then
                                if [ "$lastm" != "" ] ; then
                                        echo "</dl>"
                                fi
                                echo "<dt><img src=magenta.gif width=13 height=16 alt=\"*\""
                                echo ">&nbsp;<b>$m</b><dd><DL>" | sed 's/ /\&nbsp;/g'
                        fi
                        lastm=$m
                        f=`dirname $f`/`basename $f .menu`
                        [ ! -f $f.txt ] || f=$f.txt
                        [ ! -f $f.html ] || f=$f.html
                        echo "<dt><img src=blue.gif width=13 height=16 alt=\"*\">&nbsp;<a href=$f target=text"
                        echo ">$n</a>" | sed 's/ /\&nbsp;/g'
                done
                echo "</dl></body>"
        ) > $TOPDIR/docs/packages/menu.html
        sed 's/target=text//' <  $TOPDIR/docs/packages/menu.html > $TOPDIR/docs/packages/index.html
        cat > $TOPDIR/docs/packages/frame.html <<EOF
            <html><head><title>Midori Linux Documentation</title></head>
            <frameset border=1 cols=300,* frameborder=1 framespacing=0 >
               <frame name=menu src=menu.html scrolling=yes>
               <frame name=text src=empty.html scrolling=yes>
            </frameset>
EOF
        cat > $TOPDIR/docs/packages/empty.html <<EOF
         <html><head><title>Midori Linux Documentation</title></head><body bgcolor=ffffff></body>
EOF

}

function ml_installconfig_default() {
        local i j n ch
        [ "$KERNELOPT_REMOVE" = "" ] || mli_remove_config $IMAGE/kernel.config $KERNELOPT_REMOVE
        [ "$KERNELMOD_REMOVE" = "" ] || mli_remove_config $IMAGE/kernel.modules $KERNELMOD_REMOVE
        [ "$KERNELOPT" = "" ] || mli_add_config $IMAGE/kernel.config $KERNELOPT
        [ "$KERNELMOD" = "" ] || mli_add_config $IMAGE/kernel.modules $KERNELMOD
        [ "$BUSYBOXR" = "" ] || mli_add_config $IMAGE/busybox.root $BUSYBOXR
        [ "$BUSYBOXU" = "" ] || mli_add_config $IMAGE/busybox.usr $BUSYBOXU
        [ "$CLEANIMAGE_OPTIONS" = "" ] || mli_add_config $IMAGE/cleanimage_options $CLEANIMAGE_OPTIONS
        cd $BUILD/$PACKAGE
        if [ "$DOCFILES" != "" ] ; then
                ch=0
                mkdir -p $TOPDIR/docs/packages
                for i in $DOCFILES
                do
                        j=`basename $i`
                        if cmp -s $i $TOPDIR/docs/packages/$j ; then
                                :
                        else
                                cp -f $i $TOPDIR/docs/packages
                                chmod 555 $TOPDIR/docs/packages/`basename $i`
                                ch=1
                        fi
                done
                [ $ch = 0 ] || mli_create_index
        fi
}

# copy the tests into the correct directory
function ml_installroot_default() {
        if [ "$RUNTIMETESTS" != "" ] ; then
                mkdir -p $CACHE/runtime
                rm -rf $CACHE/runtime/${PACKAGE}__*
                for i in $RUNTIMETESTS ; do
                        n=`basename $i`
                        cd $BUILD/$PACKAGE/tests/runtime
                        if [ -d $CACHE/runtime/${PACKAGE}__$n ] ; then
                                echo "there are two tests with name `basename $i` for package $PACKAGE"
                                exit 1
                        fi
                        mkdir $CACHE/runtime/${PACKAGE}__$n
                        cp -a $i/* $CACHE/runtime/${PACKAGE}__$n
                done
        fi
        cd $BUILD
}

# clean up everything
function ml_clean_default() {
        rm -rf $PACKAGE
}

# remove all markers AFTER the one passed as $1
# build internal only, don't use
function mli_remove_marker() {
        local remove i
        remove=0
        for i in $ALL_FUNCTIONS
        do
                if [ $i = $1 ] ; then
                        remove=1
                elif [ $remove = 1 ] ; then
                        rm -f $BUILD/$PACKAGE/.$i
                fi
        done
}

# arg 1: if emtpy, do everything (but only if necessary)
#        otherwise, $1 and $2 could be:
#        force f: force a redo of function f the next time build is executed
#        upto f:  do everything up to and including f
function build() {
	ml_log build "$@"
        local F size_r size_u ml_i n opt files redo ml_j size_nr size_nu rd ud
        export ALL_FUNCTIONS=$FUNCTIONS
        case $1 in
        force)  rm -f $BUILD/$PACKAGE/.$2 ; return 0 ;;
        upto)
                set +e
                F=`expr " $FUNCTIONS " : "\(.* $2 \)"`
                set -e
                if [ "$F" = "" ] ; then
                        echo '`'"$2' is an unknown function, aborting"
                        exit 1
                fi
                FUNCTIONS="$F"
                ;;
        ?*)     FUNCTIONS="$@"
                ;;
        *)      ;;
        esac

        mkdir -p $BUILD
        set +e
        size_r=""
        if [ "$COLLECT_STATS" = "yes" -a "$STATISTICS" = "yes" -a -d $ROOT -a -d $ROOT/usr ] ; then
                size_r=`du -s -k $ROOT | awk '{ print $1 }'`
                size_u=`du -s -k $ROOT/usr | awk '{ print $1 }'`
                touch $BUILD/$PACKAGE/.now # we gamble that the first file installed will have another timestamp...
                if [ "$size_u" = "" ] ; then size_u=0 ; fi
                let size_r=$size_r-$size_u
        fi
        set -e

        for ml_i in $FUNCTIONS
        do
                cd $BUILD
                if expr " $EXECUTE_ALWAYS " : ".* $ml_i " > /dev/null ; then
                        ml_$ml_i
			ml_log $ml_i done
                else
                        n=`echo $ml_i | tr '[a-z]' '[A-Z]'`
                        opt=`eval echo '\$'${n}_OPTIONS`
                        files=`eval echo '\$'${n}_FILES`
                        redo=0
                        if [ -f $BUILD/$PACKAGE/.$ml_i ] ; then
                                for ml_j in $files
                                do
                                        if [ ! -f $ml_j -o $ml_j -nt $BUILD/$PACKAGE/.$ml_i ] ; then
                                                redo=1
                                        fi
                                done
                                if [ "`cat $BUILD/$PACKAGE/.$ml_i`" != "$opt" ] ; then
                                        redo=1
                                fi
                        else
                                redo=1
                        fi
                        if [ $redo = 1 ] ; then
                                if expr " $DANGEROUS_FUNCTIONS " : ".* $ml_i " > /dev/null ; then
                                        if [ -f $BUILD/$PACKAGE/.precious ] ; then
                                                echo
                                                echo "Your package $PACKAGE would overwrite a build directory"
                                                echo "that has been marked with a .precious file, aborting."
                                                echo "The package: $PACKAGE"
                                                echo "Build directory: $BUILD/$PACKAGE"
                                                exit 1
                                        fi
                                fi
                                ml_$ml_i
				ml_log $ml_i done
                                echo -n $opt > $BUILD/$PACKAGE/.$ml_i
                                mli_remove_marker $ml_i
                        fi
                fi
        done

        if [ "$size_r" != "" -a "$STATISTICS" = "yes" ] ; then
                set +e
                size_nr=`du -s -k $ROOT | awk '{ print $1 }'`
                size_nu=`du -s -k $ROOT/usr | awk '{ print $1 }'`
                if [ "$size_nu" = "" ] ; then size_nu=0 ; fi
                let size_nr=$size_nr-$size_nu
                set -v
                if [ "$size_r $size_u" != "$size_nr $size_nu" ] ; then
                        ml_log "files/directories added/touched:
" "`cd $ROOT ; find . -newer $BUILD/$PACKAGE/.now -ls`"
                        let rd=$size_nr-$size_r
                        let ud=$size_nu-$size_u
                        ml_log "size: $rd KB / $ud KB"
                fi
                set -e
        fi

        ml_reset
}


# extract and build an *.mlz package
# $1: mlz package, can be a full path/package, but everything but the name/version
#     will be ignored and the package as defined by the configurator will be installed.
#     The version is optional, but if it does not match installation will be aborted.
# $1 and $2: optional, arguments are passed to build script
function ml_extract() {
        local val res bn bd o m d package version var filevar file

        # the regex MUST be the same as the one used in the configurator (ML.pm), except
        # for the ending (no .mlz)
        m=`basename $1 .mlz`
        package=`echo $m | perl -pi -e 's/^(.*\/)?([A-Za-z][A-Za-z0-9+.-]*?)(-([0-9][^_]*))?(_(.*))?$/\2/'`
        version=`echo $m | perl -pi -e 's/^(.*\/)?([A-Za-z][A-Za-z0-9+.-]*?)(-([0-9][^_]*))?(_(.*))?$/\4/'`
        var=`echo $package | tr '[a-z]+.-' '[A-Z]___'`_SRC
        filevar=`echo $package | tr '[a-z]+.-' '[A-Z]___'`_SRC_PACKAGE
        val=`eval echo '\$'$var`
        file=`eval echo '\$'$filevar`

        if [ "$version" != "" -a "$package-$version" != "$val" ] ; then
                ml_extract_warn="$var==$res  -->  ignoring package $1"
		return
        fi

	pn=`basename $file .mlz`
        bn=`echo $pn | sed 's/_[a-zA-Z.0-9-]*\(internal\)\?$//'`
	bd=$BUILD/$bn
	if [ ! -f $bd/.$bn -o $file -nt $bd/.$bn ] ; then
                o=
                m=`md5sum $file | sed 's/ .*//'`
                d="$bd/.$bn"
                [ ! -f "$d" ] || o=`cat $d`
                if [ "$m" != "$o" ] ; then
                        if [ -f $bd/.precious ] ; then
                                echo "Your new package $pn would overwrite a build directory"
                                echo "that has been marked with a .precious file, aborting."
                                echo "The package: $file"
                                echo "Build directory: $bd"
                                exit 1
                        fi
                        rm -rf $bd ;
                        ml_log untarring package $file
                        tar xzf $file -C $BUILD ;
			md5sum $file | sed 's/ .*//' > $d
                fi
	fi
        shift
	ml_extract_installing=1
        (
                set +o xtrace
                set -e
                export PACKAGE=$bn
                . $SCRIPTS/functions.sh
                . $bd/build
                build "$@"
        )
}


# compile a package (usually a library) that is necessary
# in order to build another package
function ml_install() {
        if [ "$2" = "" ] ; then
                ml_extract $1 upto installlocal
        else
                ml_extract "$@"
        fi
        if [ "$ml_extract_installing" != 1 ] ; then
                echo "cannot install package $1, aborting"
                echo "$ml_extract_warn"
                exit 1
        fi
}


# check if an option is set correctly, abort otherwise
# $1: option name
# $2: required setting
function ml_required_option() {
        local opt
	opt=`eval echo '\$'$1`
	if [ "$opt" != "$2" ] ; then
		ml_log "ml_required_option $1==$2 failed, as $1=$opt"
                set +x
		echo "in order to install package $PACKAGE "
		echo "you need to set \"$1\" to \"$2\" in your config file"
		exit 1;
	fi
}

# check if a certain package will be installed and
# run an ml_install on it.
# $1: package name (linux perl gdbm realplayer
# $2: (option) version number (regexp)
function ml_required_package() {
        local mlreg mlr mlrv
        mlreg=$2
        if [ "$2" = "" ] ; then mlreg=".*" ; fi
        mlr=`echo $1_SRC | tr '[a-z]+.-' '[A-Z]___'`
        mlrv=`eval echo '\$'$mlr`
        if [ "$mlrv" = "" ] ; then
                echo "The required package '$1$mlreg' cannot be found."
                exit 1
        fi
        if [ "$mlrv" = "no" ] ; then
                echo "The required package '$1$mlreg' has been disabled."
                exit 1
        fi
        if expr "$mlrv" : "$1-$mlreg" > /dev/null ; then
                :
        else
                echo "Although the package '$1' will be installed, its the wrong version ('$mlrv' does not match '$mlreg')"
                exit 1
        fi
        ml_install $mlrv $BUILD_OPTIONS
}

# make sure that a certain package is not installed
# $1: package name (linux perl gdbm realplayer
function ml_conflicting_package() {
        local mlr mlrv
        mlr=`echo $1_SRC | tr '[a-z]+.-' '[A-Z]___'`
        mlrv=`eval echo '\$'$mlr`
        if [ "$mlrv" != "no" -a "$mlrv" != "" ] ; then
                echo "The conflicting package $mlrv will be installed, aborting."
                exit 1
        fi
}


# Abort installation of a package if another one is not
# installed, but without an error, otherwise install that package
# $1: package name (linux perl gdbm realplayer
# $2: (optional) version number (regexp)
function ml_check_for_package() {
        local mlreg mlr mlrv
        mlreg=$2
        if [ "$2" = "" ] ; then mlreg=".*" ; fi
        mlr=`echo $1_SRC | tr '[a-z]+.-' '[A-Z]___'`
        mlrv=`eval echo '\$'$mlr`
        if [ "$mlrv" = "" -o "$mlrg" = "no" ] ; then
                exit 0
        fi
        if expr "$mlrv" : "$1$mlreg" > /dev/null ; then
                ml_install $mlrv $BUILD_OPTIONS
        else
                exit 0
        fi
}

# Check if a package will be installed and return 0
# if this is the case, 1 otherwise
# $1: package name
# $2: version (optional)
function ml_is_package_installed() {
        local mlreg mlr mlrv
        mlreg=$2
        if [ "$2" = "" ] ; then mlreg=".*" ; fi
        mlr=`echo $1_SRC | tr '[a-z]+.-' '[A-Z]___'`
        mlrv=`eval echo '\$'$mlr`
        if [ "$mlrv" = "" -o "$mlrg" = "no" ] ; then
                return 1
        fi
        if expr "$mlrv" : "$1$mlreg" > /dev/null ; then
                return 0
        else
                return 1
        fi
}



# Install mlconfig files into an mlconfig directory.
# $1: partition to copy the files in. The files will actually
#     be copied into $1/lib/mlconfig/$PACKAGE. If $1=="/" &&
#     HTTPD_ROOT==no, $1 will be set to /usr
# $2-n: files to copy
function ml_mlconfig() {
        local dir i
        dir=$1
        shift

        if [ "$dir" = '/' -a "$MLCONFIG_ROOT" = no ] ; then
                dir=/usr
        fi
        if [ "$dir" != "/" -a "$dir" != /usr ] ; then
                mkdir -p $ROOT/usr/lib/mlconfig
                touch $ROOT/usr/lib/mlconfig/DIRS
                { cat DIRS ; echo  $dir/lib/mlconfig ; } | sort -u >  $ROOT/usr/lib/mlconfig/DIRS.$$
                mv $ROOT/usr/lib/mlconfig/DIRS.$$ $ROOT/usr/lib/mlconfig/DIRS
        fi
        mkdir -p $ROOT/$dir/lib/mlconfig/$PACKAGE
        for i do
                cp -a $i $ROOT/$dir/lib/mlconfig/$PACKAGE
        done
}

# SYNOPSIS
#   fix_ar_timestamps [directory list]
# DESCRIPTION
#   changes timestamps in "ar" archives to the epoch, creates some
#   temporary subdirectories in the current working directory
function fix_ar_timestamps() {
        local ar
	for ar in `find "$@" -type f -name \*.a | xargs egrep -l '^!<arch>$'`
	do
		mkdir archive-tmp.$$
		cd archive-tmp.$$
		ar x $ar
		rm -f $ar
		touch -t 197001010000.00 *
		chmod 0444 *
		chown 0:0 *
		ar r $ar *
		sed -e ':s
2 {
  /^\/ *0*[1-9]/ {
    s/^\(\/ *0*\)[1-9]/\10/
    bs
  }
}' $ar > $ar.tmp
		mv $ar.tmp $ar
		cd ..
		rm -rf $ar.tmp archive-tmp.$$
	done
}

# SYNOPSIS
#   fix_gzip_timestamps [directory list]
# DESCRIPTION
#   removes the original name and timestamp from "gzip" files and also
#   uses maximum compression
function fix_gzip_timestamps() {
	local gzip

	gzip=/tmp/gzip.$$.$RANDOM
	find "$@" -type f -name \*.gz > $gzip
	xargs gunzip -f < $gzip
	sed 's/\.gz$//' $gzip | xargs gzip -n9
	rm -f $gzip
}

# copy a file from the $ETC directory into the root tree.
# If the file does not exist, but a default version of it does,
# then copy the default version AND create the file by using
# the default data.
# $1 filename (relative to $ETC)
# $2 target filename (relative to $ROOT)
# $3 permission (octal)
# $4 owner:group [optional, default is root.root]
function ml_copy_from_etc() {
        local ml_cfe_filename ml_cfe_target ml_cfe_perm ml_cfe_owner
        ml_cfe_filename=$1
        ml_cfe_target=$2
        ml_cfe_perm=$3
        ml_cfe_owner=$4

        [ "$ml_cfe_owner" != "" ] || ml_cfe_owner="root.root"

        if [ ! -f $ETC/$ml_cfe_filename ] ; then
                if [ ! -f $ETC/$ml_cfe_filename.default ] ; then
                        echo "cannot find file $ETC/$ml_cfe_filename or $ETC/$ml_cfe_filename.default"
                        exit 1
                fi
                cp $ETC/$ml_cfe_filename.default $ETC/$ml_cfe_filename
        fi
        cp $ETC/$ml_cfe_filename $ROOT$ml_cfe_target
        chmod $ml_cfe_perm $ROOT$ml_cfe_target
        chown $ml_cfe_owner $ROOT$ml_cfe_target
}

# copy a file to the root directory while replacing some vital strings.
# All strings are of the form @XX@ and are replaced with the variable $XX.
# $1: input file
# $2: output file
#function ml_copy() {
#        sed -e "s,@CONFIG_WRITE@,$CONFIG_WRITE," \
#            -e "s,@CONFIG_DIR@,$CONFIG_DIR," \
#            -e "s,@IMAGE_DEVICE@,$IMAGE_DEVICE," \
#            -e "s,@PACKED_PARTITION@,$PACKED_PARTITION," \
#            -e "s,@CONFIG_SAVE@,$CONFIG_SAVE," \
#            -e "s,@CONFIG_PARTITION@,$CONFIG_PARTITION," $1 > $2
#}

# Same as ml_copy() but a better name, and it also strips out the "set -x"
# if VERBOSE_SCRIPTS isn't set to "yes".

function ml_transpose() {
       [ $VERBOSE_SCRIPTS = yes ] || verbose='set -x'
       [ -d `dirname $2` ] || mkdir -p `dirname $2`
        sed -e "s,@CONFIG_WRITE@,$CONFIG_WRITE,g" \
            -e "s,@CONFIG_DIR@,$CONFIG_DIR,g" \
            -e "s,@IMAGE_DEVICE@,$IMAGE_DEVICE,g" \
            -e "s,@PACKED_PARTITION@,$PACKED_PARTITION,g" \
            -e "s,@CONFIG_SAVE@,$CONFIG_SAVE,g" \
            -e "s,@CONFIG_PARTITION@,$CONFIG_PARTITION,g" \
            -e "s,@CONFIG_HTTPD_PATH@,$CONFIG_HTTPD_PATH,g" \
            -e "s,@WATCHDOG_TIMER@,$WATCHDOG_TIMER,g" \
            -e "s,@SET_ACTIVE_ARGUMENT@,$SET_ACTIVE_ARGUMENT,g" \
            -e "s,@TELNET_DAEMON_PORT@,$TELNET_DAEMON_PORT,g" \
            -e "s,@XDPI@,$XDPI,g" \
            -e "s,@DPMS@,$DPMS,g" \
            -e "s,@MLCONFIG_PORT@,$MLCONFIG_PORT,g" \
            -e "s,@MAINDIR@,$MAINDIR,g" \
            -e "s,@NGEO@,$NGEO,g" \
            -e "s,@UNAME_MINUS_R@,$UNAME_MINUS_R,g" \
            -e "s,@TEST_SUITE_KEY@,$TEST_SUITE_KEY,g" \
            -e "s,@LOGD_OPTIONS@,$LOGD_OPTIONS,g" \
            -e "s,@XTLONG@,$XTLONG,g" \
            -e "s,@SPAWN_SHELL@,$SPAWN_SHELL,g" \
            -e "s,@HDPARM_FLAGS_HDA@,$HDPARM_FLAGS_HDA,g" \
            -e "s,@HDPARM_FLAGS_HDB@,$HDPARM_FLAGS_HDB,g" \
            -e "s,@HDPARM_FLAGS_HDC@,$HDPARM_FLAGS_HDC,g" \
            -e "s,@HDPARM_FLAGS_HDD@,$HDPARM_FLAGS_HDD,g" \
           -e "s,$verbose,,g" $1 > $2
       [ "$3" = "" ] || chmod $3 $2
}

# use native toolchain 

function ml_native() {
unset GCC	|| true
unset CC	|| true
unset CFLAGS	|| true
unset LDFLAGS	|| true
unset AS	|| true
unset AR	|| true
unset RANLIB	|| true
}

# reset variables and functions
ml_reset

if [ "$DEBUG" = yes ] ; then
        set -o xtrace
fi
